% esempio di utilizzo della stima LS per regressione polinomiale
% i modelli da stimare sono polinomiali al piu' di grado 5
% 
% utilizza la function "regressioneLS_LOOCV.m" per la stima parametrica ai
% minimi quadrati dei coefficienti dei vari modelli polinomiali, anche
% applicando la tecnica di LOO-CV (Leave One Out Cross-Validation)
%
% corso Sistemi Dinamici a.a. 2021-2022
% G. Fenu
%

clc
clear all
close all

% sistema "vero": relazione polinomiale tra i dati X ed Y
N_max = 5; % grado del polinomio della relazione "vera" tra i dati

N = randi([2 N_max+1]);
coeff_v = -10 + 20 * rand(N,1); % i coefficienti del sistema "vero"

% varianza del rumore di misura
sigma_n = 0.01+ 1.98*rand; % in ]0 ; 2[

% i dati misurati
Ndati = randi([100, 1000]); % tra 100 e 1000
Xmin = -2; Xmax = 3;
X_i = Xmin + (Xmax-Xmin)*rand(Ndati,1);
rumore = sqrt(sigma_n) * randn(Ndati,1); 
% rumore bianco a valore atteso 0 e varianza sigma_n^2
Y_i = polyval(coeff_v,X_i) + rumore;

% grafico dei dati
figure;plot(X_i, Y_i,'bo'); grid on;

soluzione = regressioneLS_LOOCV(X_i, Y_i, 2*N_max);
% test con modelli polinomiali di ordine fino a 2*Nmax
% grafici in uscita con il fit di ciascun modello, gli errori di
% regressione, l'autocorrelazione degli errori, l'errore quadratico medio
disp('==================================================================');
messaggio = sprintf('Modello generatore dei dati: polinomio di grado %d',N-1);
msg2 = [];
for jj=1:N
    msg2 = [msg2 sprintf('(%.5g)*x^',coeff_v(jj)) num2str(N-jj) '+'];
end % for
msg2 = strrep(msg2,'*x^0+',''); % cancello caratteri inutili

disp(messaggio);
disp(msg2);
disp('------------------------------------------------------------');
msg3 = sprintf('Regressione polinomiale fino al grado %d',2*N_max);
disp(msg3);
disp('------------------------------------------------------------');
msg3 = sprintf('varianza del rumore (bianco) di misura %g',sigma_n);
disp(msg3);
disp('------------------------------------------------------------');

gradoLS_loo = 0; bestLS_loo = inf;
gradoLS = 0; bestLS = inf;
for ii=1:2*N_max
    test_loo = soluzione{ii}.loofit;
    if (test_loo < bestLS_loo)
        gradoLS_loo = ii;
        bestLS_loo = test_loo;
    end % if
    test_LS = soluzione{ii}.bestfitLS;
    if (test_LS < bestLS)
        gradoLS = ii;
        bestLS = test_LS;
    end % if
end % for ii
disp('*******************************************************************');
msg4 = sprintf('Migliore approssimatore LS trovato: polinomio di grado %d', gradoLS);
disp(msg4);
msg5 = [];
Nl = gradoLS+1;
for jj=1:Nl
   msg5 = [msg5 sprintf('(%.5g)*x^',soluzione{gradoLS}.parametri(jj)) num2str(Nl-jj) '+'];
end % for
msg5 = strrep(msg5,'*x^0+',''); % cancello caratteri inutili
disp(msg5);

if (gradoLS ~=(N-1)), disp('**** OVERFITTING! ****'); end % if
disp('*******************************************************************');

msg4 = sprintf('Migliore approssimatore LS-LOOCV trovato: polinomio di grado %d', gradoLS_loo);
disp(msg4);
msg5 = [];
Nl = gradoLS_loo+1;
for jj=1:Nl
   msg5 = [msg5 sprintf('(%.5g)*x^',soluzione{gradoLS_loo}.parametri(jj)) num2str(Nl-jj) '+'];
end % for
msg5 = strrep(msg5,'*x^0+',''); % cancello caratteri inutili
disp(msg5);

if (gradoLS_loo ~=(N-1)), disp('**** OVERFITTING! ****'); end % if

disp('==================================================================');
