function soluzione = regressioneLS(X, Y, N)
% test con modelli polinomiali di ordine fino a N
% grafici in uscita con il fit di ciascun modello, gli errori di
% regressione, l'autocorrelazione degli errori, l'errore quadratico medio
% 
% NB X(i) ed Y(i) sono scalari
%
% corso Sistemi Dinamici a.a. 2021-2022
% G. Fenu
%

matrice_regressori = ones(length(X),1);
termini_noti = reshape(Y, length(Y),1); % cosi' vettore colonna

[Xs idx]= sort(X); 
hf_fit = figure('Name','fitting ...');
hf_err = figure('Name','errori...');
hf_xcorr = figure('Name','autocorrelazione degli errori..');
for grado = 1:N
    matrice_regressori = [reshape(X, length(X),1) .^grado , ...
        matrice_regressori];
    parametriLS = matrice_regressori \ termini_noti;
    modello = polyval(parametriLS,X);
    figure(hf_fit);subplot(N,1,grado);
    plot(X, Y, 'bo'); hold on;
    plot(Xs, modello(idx), 'r');grid on; hold off;
    
    errori = Y - modello;
    figure(hf_err);subplot(N,1,grado);
    plot(Xs, errori(idx), 'b');
    
    [xcorr_err err_lags]= xcorr(errori, floor(length(X)/4));
    figure(hf_xcorr);subplot(N,1,grado);
    plot(err_lags, xcorr_err);
    grid on;
    
    soluzione{grado}.parametri = parametriLS;
    soluzione{grado}.errori = errori;
    soluzione{grado}.bestfit = mean(errori.^2);
    bestfit(grado) = soluzione{grado}.bestfit;
    clear modello errori parametriLS
end % for

figure('Name','LS best fit');
hbar = bar(bestfit);
