function soluzione = regressioneLS_LOOCV(X, Y, N)
% test con modelli polinomiali di ordine fino a N
% grafici in uscita con il fit di ciascun modello, gli errori di
% regressione, l'autocorrelazione degli errori, l'errore quadratico medio
%
% NB X(i) ed Y(i) sono scalari
%
% corso Sistemi Dinamici a.a. 2021-2022
% G. Fenu
%


nX = length(X); % quanti punti?
[Xs idx]= sort(X); % li riordino

for grado = 1:N % esploro tutti i modelli
    soluzione{grado}.parametri = [];% parametri del modello
    soluzione{grado}.errori = [];   % errore = osservazione-modello
    soluzione{grado}.bestfitLS = 0; % errore quadratico medio
    soluzione{grado}.loofit = 0;    % il valore dell'indice LOO-CV
    % inizializzo
            
    for id_cv = 1 :nX % ciclo che esegue LOO-CV
        % LOO-CV: Leave One Out Cross-Validation
        looX = X; looY = Y;
        looX(id_cv)=[]; looY(id_cv) = []; % tolgo 1 dato
        termini_noti = reshape(looY, length(Y)-1,1); % cosi' vettore colonna
        matrice_regressori = ones(length(X)-1,1);
        for j=1:grado
            matrice_regressori = [reshape(looX, length(looX),1) .^j , ...
            matrice_regressori];
        end % for j
        parametriLS = matrice_regressori \ termini_noti; % risolvo il problema LS
        modello = polyval(parametriLS,X); % calcolo il polinomio
        
        % figure(hf_fit);subplot(N,1,grado);
        %plot(X, Y, 'bo'); hold on;
        %plot(Xs, modello(idx), 'r');grid on; hold off;

        errori = Y - modello;
        %figure(hf_err);subplot(N,1,grado);
        %plot(Xs, errori(idx), 'b');

        %[xcorr_err err_lags]= xcorr(errori, floor(length(X)/4));
        %figure(hf_xcorr);subplot(N,1,grado);
        %plot(err_lags, xcorr_err);
        %grid on;

        soluzione{grado}.parametri = parametriLS; % i coefficienti del modello trovato
        soluzione{grado}.errori = errori; % gli scarti tra modello e misure
        soluzione{grado}.bestfitLS = mean(errori.^2); % errore quadratico medio
        soluzione{grado}.loofit = soluzione{grado}.loofit + ...
            (Y(id_cv) - polyval(parametriLS,X(id_cv))).^2; % il valore dell'indice LOO-CV
        bestfit(grado) = soluzione{grado}.loofit;
    
        
    
   
        clear modello errori parametriLS
    end % for grado
end % for id_cv
bestfit = bestfit ./ nX; % scalo l'indice LOO-CV
% figure('Name','LS LOO best fit');
% hbar = bar(bestfit);
